﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;

namespace APISampleWebSite
{
	public static class TokenProvider
	{
		public static bool HasTokenExpired(string expiresAt)
		{
			bool value = false;

			DateTime expiry;
			if (DateTime.TryParseExact(expiresAt, "o", CultureInfo.InvariantCulture,		
                                    DateTimeStyles.AdjustToUniversal, out expiry))
			{
				if (DateTime.UtcNow > expiry)
				{
					value = true;
				}
			}

			return value;
		}

		/// <summary>
		/// Attempt to renew access token
		/// </summary>
		/// <param name="session">The session</param>
		/// <returns>True if refresh was successful</returns>
		public static string RenewAccessToken(ClaimsIdentity identity, string refreshToken)
		{
			string token = string.Empty;

			// If a refresh token wasn't issued then we can't refresh
			if (string.IsNullOrWhiteSpace(refreshToken)) { return token; }

			string url = Settings.Issuer + "oauth/token";

			//Construct request (POST) to authorisation server
			HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);
			request.Method = "POST";

			request.ContentType = "application/x-www-form-urlencoded;charset=UTF-8";

			//Add parameters to body of the request.
			byte[] requestParameters = System.Text.Encoding.UTF8.GetBytes(
				"grant_type=refresh_token" +
				"&client_id=" + Settings.ClientID + 
				"&client_secret=" + Settings.ClientSecret + 
				"&refresh_token=" + refreshToken);


			request.GetRequestStream().Write(requestParameters, 0, requestParameters.Length);

			HttpWebResponse response;
			try
			{
				//Send request
				response = (HttpWebResponse)request.GetResponse();

				//Read response.
				string json = (new StreamReader(response.GetResponseStream())).ReadToEnd();

				TokenResponse tokenResponse = Newtonsoft.Json.JsonConvert.DeserializeObject<TokenResponse>(json);

				//Client should persist new tokens here.
				UpdateClaims(identity, tokenResponse);
			
				token = tokenResponse.AccessToken;
			}
			catch (WebException we)
			{
				//We have to catch WebException because the authorisation server
				//will return a 400 response for an unsuccessful call.

				response = (HttpWebResponse)we.Response;

				string responseBody = (new StreamReader(response.GetResponseStream())).ReadToEnd();
				System.Diagnostics.Debug.WriteLine("RenewAccessToken, webException: " + responseBody);
			}
			catch (Exception ex)
			{
				System.Diagnostics.Debug.WriteLine("RenewAccessToken: " + ex.Message);
			}

			return token;
		}

		public static void UpdateClaims(ClaimsIdentity identity, TokenResponse tokenResponse)		
		{
			if (identity != null)
			{
				if (!string.IsNullOrEmpty(tokenResponse.ExpiresIn))
				{
					int value;
					if (int.TryParse(tokenResponse.ExpiresIn, NumberStyles.Integer, CultureInfo.InvariantCulture, out value))
					{
						// value = 120;		// reinstate this line to exercise token refresh after two minutes
						var expiresAt = DateTime.UtcNow + TimeSpan.FromSeconds(value);
						// https://www.w3.org/TR/xmlschema-2/#dateTime
						// https://msdn.microsoft.com/en-us/library/az4se3k1(v=vs.110).aspx
						string expiryTime = expiresAt.ToString("o", CultureInfo.InvariantCulture);
						identity.SetClaim(ClaimTypes.AccessTokenExpiry, expiryTime);
					}
				}

				identity.SetClaim(ClaimTypes.AccessToken, tokenResponse.AccessToken);
				if (string.IsNullOrEmpty(tokenResponse.RefreshToken) == false)
				{
					identity.SetClaim(ClaimTypes.RefreshToken, tokenResponse.RefreshToken);
				}
			}
		}
	}

	public class TokenResponse
	{
		[Newtonsoft.Json.JsonProperty("access_token")]
		public string AccessToken {get; set;}

		[Newtonsoft.Json.JsonProperty("refresh_token")]
		public string RefreshToken { get; set; }

		[Newtonsoft.Json.JsonProperty("expires_in")]
		public string ExpiresIn { get; set; }
	}
}
